;*============================================================================
;*      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
;*      Copyright (c) $Date: 94/05/02 15:45:09 $ - All rights reserved.
;*
;*      No part of this document may be reproduced in any form
;*      without the express written consent of MODICON, Inc.
;*
;*=fdoc=======================================================================
;*
;* File:        desqv.asm
;*
;* Functions:
;* Name                 Description
;* -------------------- -----------
;*	dv_get_version			Get the DESQview version number.
;*	DV_GET_VIDEO_BUFFER	Returns DESQview's alternate video buffer, if present.
;*	API_CALL					Entry into DESQview.
;*	DV_PAUSE					Swaps out your task.
;*	DV_BEGIN_CRITICAL		Disables DESQview swapping.
;*	DV_END_CRITICAL		Enables DESQview swapping
;*
;* Notes:       DESQview interfaces.
;*
;* Author:      
;*
;* RCSID:
;* $Header: /usr2/dpapadop/pcmbp/dos/mbphost/RCS/desqv.asm,v 3.34 94/05/02 15:45:09 dpapadop R3_41 $
;*
;* RCS:
;* $Source: /usr2/dpapadop/pcmbp/dos/mbphost/RCS/desqv.asm,v $
;* $Revision: 3.34 $
;* $Author: dpapadop $
;* $State: R3_41 $
;* $Locker: $
;*
;* Revision History Log:
;*
;* $Log: desqv.asm,v $
;* Revision 3.34  94/05/02 15:45:09  dpapadop
;* Made several functions public.
;* Changed names of several functions to accomodate case sensitivity.
;; Revision 3.33  1994/03/11  21:07:41  dpapadop
;;  Source for the MB+ DOS device driver with PCMCIA support
;;
;*
;*=fdoc=====================================================================*/

		TITLE	DESQview Interfaces

IFDEF   TURBO
        version m510
ENDIF

;        .286P                           ; enable protected-mode instr.
IFDEF mem_s
.seq

_DATA segment word public 'DATA'
_DATA ends

CONST segment word public 'CONST'
CONST ends

_BSS segment word public 'BSS'
_BSS ends

STACK segment para stack 'STACK'
STACK ends

_TEXT segment word public 'CODE'
_TEXT ends

	.MODEL	small,C

; Define the Data Group for the small memory model.

DGROUP group CONST, _BSS, _DATA

	.data

ELSE
	DOSSEG
	.MODEL	LARGE,C

; N O T I C E	<<==========
;  This file requires the MASM 5.1 assembler

;       .FARDATA?
		.DATA?
ENDIF

IN_DV   DB      1

		.CODE

		
;*=doc========================================================================
;*
;* Function:    dv_get_version
;*
;* Description: 	Returns in AH/Al the DESQview major/minor version numbers,
;*						and sets up the IN_DV variable for later use.
;*
;* Returns:     Returns 0 in AX if DESQview isn't there.
;*
;* Parameters:  None
;*
;* Notes:       
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;* Papadopoulos		01/23/94		Ported to Borland 3.1
;*=doc======================================================================*/


IFDEF TASM
PUBLIC dv_get_version

dv_get_version PROC C   FAR  USES BX CX DX

ELSE

dv_get_version PROC FAR C  USES BX CX DX
ENDIF




        mov     cx,'DE'         ; set CX to 4445h; DX to 5351h
        mov     dx,'SQ'         ; (an invalid date)
        mov     ax,2b01h        ; DOS' set date function
        int     21h             ; call DOS
        cmp     al,0ffh         ; did DOS see it as invalid?
        je      NO_DESQVIEW     ; if so, DESQview isn't there
        mov     ax,bx           ; AH=major version; AL=minor version
;       mov     cs:IN_DV,1      ; set internal variable used by
        mov     IN_DV,1      ; set internal variable used by
        jmp     short DVGV_X    ;       other routines
NO_DESQVIEW:
        sub     ax,ax           ; return no DESQview (version 0)
DVGV_X: ret

dv_get_version ENDP

;*=doc========================================================================
;*
;* Function:    DV_GET_VIDEO_BUFFER
;*
;* Description: 	Takes the hardware video segment on the stack and returns 
;*						that segment (if DESQview is not present) or DESQview's 
;*						alternate video buffer in AX.  Sets up the IN_DV variable 
;*						for later use. Call this instead of dv_get_version if your 
;*						program writes directly to video memory.
;*
;* Returns:     x
;*
;* Parameters:  x
;*
;* Notes:       x
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;* Papadopoulos		01/23/94		Ported to Borland 3.1
;*=doc======================================================================*/



IFDEF TASM
DV_GET_VIDEO_BUFFER PROC C FAR hwseg:WORD
ELSE
DV_GET_VIDEO_BUFFER PROC    FAR C hwseg:WORD
ENDIF


        push    bp
        mov     bp,sp
        push    di
        push    es
        mov     es,hwseg        ; Put the hardware segment into ES
        call    dv_get_version  ; Returns AX=0 if not in DESQview
        test    ax,ax           ; In DV?
        jz      DVGVB_X         ; jump if not
; Since DESQview is running, get the alternate screen buffer
        mov     ah,0feh         ; DV's get buffer function
        int     10h             ; Returns ES:DI as alternate buffer
DVGVB_X:
        mov     ax,es           ; Return correct video buffer in ES
        pop     es
        pop     di
        pop     bp
        ret     2
DV_GET_VIDEO_BUFFER ENDP

;*=doc========================================================================
;*
;* Function:    API_CALL
;*
;* Description: 	This local routine takes a program interface function in BX,
;*						and makes that call to DESQview after switching onto a stack
;*						that DESQview provides for your program.
;*
;* Returns:     x
;*
;* Parameters:  x
;*
;* Notes:       x
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;*=doc======================================================================*/



API_CALL PROC NEAR USES AX

        mov     ax,101ah        ; The func to switch to DV's stack
        int     15h             ; DV's software interrupt
        mov     ax,bx           ; move the desired function to AX
        int     15h             ; Make that call
        mov     ax,1025h        ; Func to switch off of DV's stack
        int     15h             ; Make that call
        ret
API_CALL ENDP


;*=doc========================================================================
;*
;* Function:    DV_PAUSE
;*
;* Description: This routine gives up the rest of your program's time slice.
;*
;* Returns:     None
;*
;* Parameters:  None
;*
;* Notes:       x
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;*=doc======================================================================*/



DV_PAUSE PROC    FAR C 

;       cmp     cs:IN_DV,1      ; Are we in DESQview?
        cmp     IN_DV,1      ; Are we in DESQview?
        jne     DVP_X           ; if not, nothing to do
        push    bx              ; Else make the pause function call
        mov     bx,1000h        ; This is the function code
        call    API_CALL        ; Do it
        pop     bx      
DVP_X:  ret
DV_PAUSE ENDP

;*=doc========================================================================
;*
;* Function:    DV_BEGIN_CRITICAL
;*
;* Description: 	This routine tells DESQview not to slice away from your 
;*						program until you make a DV_END_CRITICAL call.
;*
;* Returns:     None
;*
;* Parameters:  None
;*
;* Notes:       x
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;*=doc======================================================================*/

IFDEF TASM
PUBLIC dv_begin_critical

dv_begin_critical PROC C   FAR  

ELSE

dv_begin_critical PROC    FAR C 
ENDIF

;       cmp     cs:IN_DV,1      ; Are we in DESQview?
        cmp     IN_DV,1      ; Are we in DESQview?
        jne     DVBC_X          ; If not, nothing to do
        push    bx              ; Else make the begin critical call
        mov     bx,101bh        ; This is the function code
        call    API_CALL        ; Do it
        pop     bx
DVBC_X: ret
dv_begin_critical ENDP

;*=doc========================================================================
;*
;* Function:    dv_end_critical
;*
;* Description: 	This routine tells DESQview that it is all right to slice
;*						away from your program again.
;*
;* Returns:     None
;*
;* Parameters:  None
;*
;* Notes:       x
;*
;* Revision History Log:
;*
;*	NAME					DATE			COMMENT
;*	-----					-----			--------
;*=doc======================================================================*/



IFDEF TASM
PUBLIC dv_end_critical

dv_end_critical PROC C   FAR  

ELSE

dv_end_critical PROC    FAR C 
ENDIF

;       cmp     cs:IN_DV,1      ; Are we in DESQview?
        cmp     IN_DV,1      ; Are we in DESQview?
        jne     DVEC_X          ; If not, nothing to do
        push    bx              ; Else make the end critical call
        mov     bx,101ch        ; This is the function code
        call    API_CALL        ; Do it
        pop     bx
DVEC_X: ret
dv_end_critical ENDP

				end
