/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) 1989, 1990 - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        libmain.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * LibMain              DLL Initialization Function
 * WEP                  DLL termination function
 *
 * Notes:       
 *
 * Author:      pmurphy
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.2 $
 * $Date: 94/07/01 19:28:29 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 * $Log: libmain.c,v $
 * Revision 1.2  94/07/01 19:28:29  dp
 * Added code to retrieve S/W interrupt from the MODICON.INI file if possible.
 * Revision 1.3  1992/04/22  11:42:43  pmurphy
 * Added globals to indicate state of MSWindows.
 *
 * Revision 1.2  1992/04/03  08:25:22  pmurphy
 * Initialize protected NCB pointer to NULL.
 *
 * Revision 1.1  90/12/07  18:43:38  pmurphy
 * Initial revision
 * 
 * Revision 1.1  90/12/07  09:53:19  pmurphy
 * Initial revision
 * 
 *
 *=fdoc=====================================================================*/

#include "windows.h"

#include <dos.h>
#include <stdio.h>

#include "netbios.h"
#include "netlib.h"

/* ========================================================================
			     GLOBAL DATA
   ======================================================================== */

DWORD dwWinFlags;		/* Window mode flags */
int fIsEnhanced;		/* Is 386 Enhanced mode of windows */

/*=doc========================================================================
 *
 * Function:    LibMain
 *
 * Description: DLL Initialization Function
 *
 * Returns:     1 if successfull, else 0
 *
 * Notes:       Required by LibEntry()
 *
 *=doc======================================================================*/

int FAR PASCAL LibMain(HANDLE hInstance, /* Instance handle */
		       WORD wDataSeg, /* Value of DS register */
		       WORD cbHeapSize,	/* Size of Local heap */
		       LPSTR lpszCmdLine) /* Pointer to command line */
{
	extern int sw_interrupt;
	int i, ret;

	char szAdapterNumber[6];
	char szBuffer[80];

	if (cbHeapSize)
   	UnlockData(0);		/* Unlock DLL data segment */

	dwWinFlags = GetWinFlags();

	fIsEnhanced = ((dwWinFlags & WF_ENHANCED) != 0);


	/*
	**	Look in the MODICON.INI file for possible entries for MB+ adapters.
	**	If an entry is found, Use the value for the SW interrupt found there.
	*/

	for (ret = 0, i = 0; (i<2) && !(ret); i++)
	{
		sprintf( szAdapterNumber, "MBP%d", i);

		ret = GetPrivateProfileString("Ports", szAdapterNumber, "",
				szBuffer, sizeof(szBuffer), "MODICON.INI");
	}
	if ((ret) && (*szBuffer != ';'))
	{
		sscanf(szBuffer, "%x ", &sw_interrupt);
	}
	else
	{
		sw_interrupt = SW_INTERRUPT;
	}

	return 1;			/* Return Successfully */
}
