;*============================================================================
;*      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
;*      Copyright (c) $Date: 94/05/02 15:44:10 $ - All rights reserved.
;*
;*      No part of this document may be reproduced in any form
;*      without the express written consent of MODICON, Inc.
;*
;*=fdoc=======================================================================
;*
;* File:        win3api.asm
;*
;* Functions:
;* Name                 	Description
;* -------------------- 	-----------
;* IsEnhancedWindows    	Determine if enhanced windows is running (Virtual 8086)
;* WinEnterCriticalSection Enters a critical section of Windows,
;*				 					no task switch will occur
;* WinEndCriticalSection 	Ends a critical section of windows.
;*
;* Notes:       
;*
;* Author:      pmurphy
;*
;*
;* RCSID:
;* $Header: /usr2/dpapadop/pcmbp/dos/mbphost/RCS/win3api.asm,v 3.34 94/05/02 15:44:10 dpapadop R3_41 $
;*
;* RCS:
;* $Source: /usr2/dpapadop/pcmbp/dos/mbphost/RCS/win3api.asm,v $
;* $Revision: 3.34 $
;* $Author: dpapadop $
;* $State: R3_41 $
;* $Locker: $
;*
;* Revision History Log:
;*
;* $Log: win3api.asm,v $
;* Revision 3.34  94/05/02 15:44:10  dpapadop
;* Made WinEnterCriticalSection and WinEndCriticalSection public.
;; Revision 3.33  1994/03/11  21:07:41  dpapadop
;;  Source for the MB+ DOS device driver with PCMCIA support
;;
;*
;*=fdoc=====================================================================*/

IFDEF   TURBO
        version m510
ENDIF

IFDEF mem_s
.seq

_DATA segment word public 'DATA'
_DATA ends

CONST segment word public 'CONST'
CONST ends

_BSS segment word public 'BSS'
_BSS ends

STACK segment para stack 'STACK'
STACK ends

_TEXT segment word public 'CODE'
_TEXT ends

	.MODEL small,c
ELSE
	.MODEL large,c
ENDIF


IFDEF TASM
PUBLIC IsEnhancedWindows
PUBLIC WinEnterCriticalSection
PUBLIC WinEndCriticalSection
ENDIF

	.CODE

 ;=doc========================================================================
 ;
 ; Function:    unsigned _cdecl _far IsEnhancedWindows(void)
 ;
 ; Description: Determine if enhanced windows is running (Virtual 8086)
 ;
 ; Returns:     0 if Enhanced windows is not running, else
 ;		major version of windows in low byte,
 ;		minor version of windows in high byte
 ;
 ; Notes:       
 ;
 ;=doc=======================================================================

IsEnhancedWindows proc far

	mov	ax,1600h		;Enhanced Windows installation check
	int	2fh

	test	al,7fh			;If 0 or 0x80 returned enhanced
	jnz	short t1  		;.Windows is not running in enhanced
	xor	ax,ax			;..mode, So return 0
	jmp	short ProcEnd
t1:
	cmp	al,1			;If 1 or 0xff is returned
	je	short t2		;.Windows 386/2.0 is running
	cmp	al,-1
	jne	ProcEnd
t2:
	mov	ax,2			;..So fake it, and say its version 2
ProcEnd:				;Otherwise the major window version
	ret				;.is in the low byte, and the minor
					;..version is in the high byte

IsEnhancedWindows	endp


 ;=doc========================================================================
 ;
 ; Function:    void _cdecl _far WinEnterCriticalSection(void)
 ;
 ; Description: Enters a critical section of Windows, no task switch will
 ;		occur
 ;
 ; Returns:     void
 ;
 ; Notes:       WinEndCriticalSection must!!! be called after the critical
 ;		portion of the code is executed.
 ;
 ;=doc=======================================================================

WinEnterCriticalSection proc far

	mov	ax,1681h		;Begin Critical section
	int	2fh
	ret

WinEnterCriticalSection	endp

 ;=doc========================================================================
 ;
 ; Function:    void _cdecl _far WinEndCriticalSection(void)
 ;
 ; Description: Ends a critical section of windows.
 ;
 ; Returns:     void
 ;
 ; Notes:       
 ;
 ;=doc=======================================================================

WinEndCriticalSection proc far

	mov	ax,1682h		;End Critical section
	int	2fh
	ret

WinEndCriticalSection	endp

	end

