/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) $Date: 94/05/02 15:17:18 $ - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        bdreset.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * main						Accepts input and sends a reset to the MB+ card
 *	syntax					Prints an informational message on the use of bdreset
 *
 * Notes:       x
 *
 * Author:      x
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.1 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 *
 *=fdoc=====================================================================*/

#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "netbios.h"
#include "netlib.h"

/*
**	externals
*/

int sw_int;

/*
** prototypes
*/
int main(int argc, char *argv[]);
void syntax(void);

/*=doc========================================================================
 *
 * Function:    main
 *
 * Description: x
 *
 * Returns:     x
 *
 * Parameters:		Syntax: BDRESET [/s] [/n]
 *						Where /s provides the s/w interrupt to use (default 0x5c)
 *						And /n is the LAN adapter number to use (default #0)
 *						Example:   BDRESET /s5d /n1
 *						Reset adapter #1, using s/w int 0x5d
 *
 * Notes:       x
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int main(argc, argv)
int argc;
char *argv[];
{
    int i;
    int adaptno = 0; /* Default */
	 int ret;

    printf("\nReset Modbus Plus board via s/w reset via NETBIOS...");

    if (argc == 1) {
        ret = ncb_reset(0);
        printf("\nAdapter #0 reset. Return = %d\n",ret);
        exit(0);
        }

    for (i = 0;  i < argc - 1;  i++) {
        if (!strnicmp(argv[i + 1], "/s", 2))
			{
            /*set s/w int to use */
            argv[i + 1][1] = toupper(argv[i+1][1]);
            if (sscanf(argv[i + 1], "/S%x", &sw_int) != 1)
				{
					fprintf(stderr, "Illegal SW interrupt value!\n");
					exit(0);
				}
				else
				{
					/*
					**	Note the use of the new set interrupt library call.
					**	There is no guarantee that the global sw_interrupt
					**	will be accessible in future releases. (4/20/94)
					*/
            	sw_int &= 0xFF;
					if (ncb_set_sw_interrupt(sw_int) != ERR_success)
					{
						fprintf(stderr, "Invalid SW interrupt value!\n");
						exit(0);
					}
				}


/*             fprintf(stderr,"\nUsing sw_interrupt 0x%02X\n\n",sw_int); */
			}
        else
			{
            if ((strnicmp(argv[i + 1], "/h", 2) == 0) || 
                (strnicmp(argv[i + 1], "-h", 2) == 0) || 
                (strnicmp(argv[i + 1], "?", 1) == 0))
				{
                    /* Syntax help */
                syntax();
                exit(0);
            }
            else
				{
               if (!strnicmp(argv[i + 1], "/n", 2))
					{
            /*set lan adapter # to use */
                    argv[i + 1][1] = toupper(argv[i+1][1]);
                    if (sscanf(argv[i + 1], "/N%d", &adaptno) != 1)
                        adaptno = 0;
                    adaptno &= 0xFF;
/*                   fprintf(stderr,"\nUsing lan adapter # %d\n\n",adaptno); */
               }
               else
					{
                    syntax();
                    exit(0);
               }
            }
         }
    }
/*  printf("\nAdapter reset, status returned was 0x%x.\n",testncbp->NCB_RETCODE); */
    ret = ncb_reset(adaptno);
    printf("\nAdapter #%d reset. Return = %d\n", adaptno, ret);

    return 0;
}

/*=doc========================================================================
 *
 * Function:    syntax
 *
 * Description: Prints an error message when main receives illegal input.
 *
 * Returns:     None
 *
 * Parameters:  None
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


void syntax()
{
    fprintf(stderr, "\nSyntax BDRESET [/s] [/n]");
    fprintf(stderr, "\nwhere /s provides the s/w interrupt to use (default 0x5c)");
    fprintf(stderr, "\nwhere /n is the LAN adapter number to use (default #0)");
    fprintf(stderr, "\nExample:   BDRESET /s5c /n0");
    fprintf(stderr, "\n           Reset adapter #0, using s/w int 0x5c");
    fprintf(stderr, "\n5c is the Recommended s/w interrupt to use!");
    fprintf(stderr, "\nDefault is adapter #0, s/w int 0x5c\n");
}					   
