/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) $Date: 94/05/02 15:17:51 $ - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        sa85off.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * main						Accepts input and shuts down the MB+ card
 *	syntax					Prints an informational message on the use of sa85off
 *
 * Notes:	This turns of the timer tick processing for the Modbus Plus
 * 			board (RESET to turn back on).
 *
 * Author:      
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.1 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 *
 *=fdoc=====================================================================*/

#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include "netbios.h"
#include "netlib.h"

int sw_int;

/*
** prototypes
*/

int main(int argc, char *argv[]);
void syntax(void);

/*=doc========================================================================
 *
 * Function:    main
 *
 * Description: 	Parses user input and sends command.
 *
 * Returns:     
 *
 * Parameters:		Syntax "SA85OFF [/s] [/n]"
 *
 *				Where /s provides the s/w interrupt to use (default 0x5c) and
 *				/n is the LAN adapter number to use (default #0)
 *
 *				Example:   SA85OFF /s5d /n1
 *          	Turn off driver for adapter #1, using s/w int 0x5d  
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int main(argc, argv)
int argc;
char *argv[];
	{
	int i;
	int adaptno = 0;
	int ret;

	printf("\nTurn off Modbus Plus driver via NETBIOS...");
	printf("\nIssue BDRESET to turn back on");
	if (argc == 1) {
		ret = ncb_sa85off(0);
		printf("\nAdapter #0 turned off. Return = %d\n", ret);
		exit(0);
		}

	for (i = 0;  i < argc - 1;  i++) {
		if (!strnicmp(argv[i + 1], "/s", 2)) {
            /*set s/w int to use */
			argv[i + 1][1] = toupper(argv[i+1][1]);
         if (sscanf(argv[i + 1], "/S%x", &sw_int) != 1)
			{
				fprintf(stderr, "Illegal SW interrupt value!\n");
				exit(0);
			}
			else
			{
				/*
				**	Note the use of the new set interrupt library call.
				**	There is no guarantee that the global sw_interrupt
				**	will be accessible in future releases. (4/20/94)
				*/
           	sw_int &= 0xFF;
				if (ncb_set_sw_interrupt(sw_int) != ERR_success)
				{
					fprintf(stderr, "Invalid SW interrupt value!\n");
					exit(0);
				}
			}
		}

		else {
			if ((strnicmp(argv[i + 1], "/h", 2) == 0) || 
				(strnicmp(argv[i + 1], "-h", 2) == 0) || 
				(strnicmp(argv[i + 1], "?", 1) == 0)) {
                    /* Syntax help */
				syntax();
				exit(0);
				}
			else {
				if (!strnicmp(argv[i + 1], "/n", 2)) {
            /*set lan adapter # to use */
					argv[i + 1][1] = toupper(argv[i+1][1]);
					if (sscanf(argv[i + 1], "/N%d", &adaptno) != 1)
						adaptno = 0;
					adaptno &= 0xFF;
/*		             fprintf(stderr,"\nUsing lan adapter # %d\n\n",adaptno); */
					}
				else {
					syntax();
					exit(0);
					}
				}
			}
		}

	ret = ncb_sa85off(adaptno);
	printf("\nAdapter #%d turned off. Return = %d\n", adaptno, ret);

	return 0;
	}

/*=doc========================================================================
 *
 * Function:    syntax
 *
 * Description: Prints an error message when main receives illegal input.
 *
 * Returns:     None
 *
 * Parameters:  None
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

void syntax()
{
	fprintf(stderr, "\nSyntax SA85OFF [/s] [/n]");
	fprintf(stderr, "\nwhere /s provides the s/w interrupt to use (default 0x5c)");
	fprintf(stderr, "\nwhere /n is the LAN adapter number to use (default #0)");
	fprintf(stderr, "\nExample:   SA85OFF /s5c /n0");
	fprintf(stderr, "\n           Turn off driver for adapter #0, using s/w int 0x5c");
	fprintf(stderr, "\n5c is the Recommended s/w interrupt to use!\n");
}
