/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) $Date: 94/07/08 20:08:52 $ - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        test4b.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * main
 *	control_c				CTLC handler
 *
 * Notes:		This test program uses the device driver to read 125 holding
 *					registers from two MODBUS slaves. 
 *
 *					In order to use this program, enter the following command:
 *
 *						A>TEST4B <address1> <address2> [dual adapter flag]
 *
 *					Where <addressx> is the MB+ address of the slave nodes we want
 *					to read from, and [dual adapter flag] indicates whether or not
 *					to use two MB+ adapter cards to communicate. (Default is one
 *					adapter - adapter 0.)
 *
 * Author:      
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.2 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 *
 *=fdoc=====================================================================*/

#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <conio.h>
#include <signal.h>
#include "netbios.h"
#include "netlib.h"


/*
**	prototypes
*/
int main(int argc, char *argv[] );
void control_c(void);

/*
**	global variables
*/

char mbuffer[256];
char path1[ 80 ];
char path2[ 80 ];
int completed;

/*=doc========================================================================
 *
 * Function:    main
 *
 * Description: 	Parses user input and gets register input.
 *
 * Returns:     
 *
 * Parameters:  	MB+ addresses of remote nodes and flag indicating number
 *						host adapters to use.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int main( argc, argv )
int argc;
char *argv[];
	{
	int first_adapter = 0;
	int second_adapter = 0;
	NCB *nd1;
	NCB *nd2;
	int ret_val;
	int i;
    int j;

    printf( "MODBUS Command/Response test.\n" );
    printf( "\"+\" Indicates First Command Issued.\n" );
    printf( "\"<\" Indicates Second Command Issued.\n" );
    printf( "\"-\" Indicates First Response Received.\n" );
    printf( "\">\" Indicates Second Response Received.\n" );

    if( argc < 3 ) {
        printf( "Usage is: A>TEST4B <first slave node> <second slave node>\n");
        exit( 1 );
    }
    ret_val = sscanf( argv[ 1 ], "%d", &i );
    ret_val += sscanf( argv[ 2 ], "%d", &j );
    if( ret_val != 2 || i < 1 || i > 64 ) {
        printf( "Node numbers must be in the range 1 to 64. " );
        exit( 1 );
    }
    
    if( signal( SIGINT, control_c ) == SIG_ERR ) {
        printf( "Unable to redirect Control-C." );
        exit( 1 );
    }
	 if (argc == 4)
		second_adapter = 1;

	if (argc == 5)
		second_adapter = first_adapter = 1;

    sprintf( path1, "DM.%d.0.0.0.0", i );
    sprintf( path2, "DM.%d.0.0.0.0", j );

	if ((nd1 = ncb_open( path1, first_adapter )) == NULL) {
		printf("Unable to open first DATA MASTER path.\n");
		exit(1);
		}
	if ((nd2 = ncb_open( path2, second_adapter )) == NULL) {
		printf("Unable to open second DATA MASTER path.\n");
		exit(1);
		}
	printf("Path %02X opened\n", nd1->NCB_NUM);
	printf("Routing info: %c%c.%d.%d.%d.%d.%d\n",
		nd1->NCB_CALLNAME[0],
		nd1->NCB_CALLNAME[1],
		nd1->NCB_CALLNAME[2],
		nd1->NCB_CALLNAME[3],
		nd1->NCB_CALLNAME[4],
		nd1->NCB_CALLNAME[5],
		nd1->NCB_CALLNAME[6]);

	printf("Path %02X opened\n", nd2->NCB_NUM);
	printf("Routing info: %c%c.%d.%d.%d.%d.%d\n",
		nd2->NCB_CALLNAME[0],
		nd2->NCB_CALLNAME[1],
		nd2->NCB_CALLNAME[2],
		nd2->NCB_CALLNAME[3],
		nd2->NCB_CALLNAME[4],
		nd2->NCB_CALLNAME[5],
		nd2->NCB_CALLNAME[6]);

    completed = 0;          /* global variable, do while zero */
    while( !completed ) {
	    if( kbhit() )
            completed = 1;
        mbuffer[0] = 0x5;	/*slave address*/
		mbuffer[1] = 0x3;	/*command*/
		mbuffer[2] = 0x0;	/*offset high*/
		mbuffer[3] = 0x0;	/*offset low*/
		mbuffer[4] = 0x0;	/*reg count high*/
		mbuffer[5] = 125;	/* reg count low*/
		printf("+");
		if (ncb_send(nd1, 6, mbuffer, 10) != 0) {   /*send first command*/
						/*send the command*/
			printf("Send error: %d.\n", nd1->NCB_RETCODE);
			ret_val = ncb_close(nd1);	/*close the path*/
			ret_val = ncb_close(nd2);	/*close the path*/
			exit(1);
			}

		mbuffer[0] = 0x5;	/*slave address*/
		mbuffer[1] = 0x3;	/*command*/
		mbuffer[2] = 0x0;	/*offset high*/
		mbuffer[3] = 0x0;	/*offset low*/
		mbuffer[4] = 0x0;	/*reg count high*/
		mbuffer[5] = 125;	/* reg count low*/
		printf("<");
		if (ncb_send(nd2, 6, mbuffer, 10) != 0) {   /*send second command*/
						/*send the command*/
			printf("Send error: %d.\n", nd2->NCB_RETCODE);
			ret_val = ncb_close(nd1);	/*close the path*/
			ret_val = ncb_close(nd2);	/*close the path*/
			exit(1);
			}

		if (ncb_receive_wait(nd1, mbuffer, 10) != 0) {
						/*try to receive*/
			printf("\nReceive error: %d.\n", nd1->NCB_RETCODE);
			}
		printf("-");

		if (ncb_receive_wait(nd2, mbuffer, 10) != 0) {
						/*try to receive*/
			printf("\nReceive error: %d.\n", nd2->NCB_RETCODE);
			}
		printf(">");
		}
	ncb_close(nd1);		/*close the path*/
	ncb_close(nd2);		/*close the path*/
	exit(0);
    return( 0 );
	}

/*=doc========================================================================
 *
 * Function:    control_c
 *
 * Description: 	This routine replaces the control-C handler supplied by DOS.
 *						When the user enters a control_C, the program will vector to
 *						this routine, which will set the completed flag to non-zero.
 *						In this test program, this will cause the infinite loop
 *						within main to terminate.
 *
 * Returns:     None
 *
 * Parameters:  Signal which caused the call. (SIGINT)
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

void control_c()
{
    signal( SIGINT, SIG_IGN );          /* disable control-c */
    completed = 1;                      /* will cause main loop to complete */
    signal( SIGINT, control_c );        /* reset control-c handler */
}


