/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) $Date: 94/05/02 15:09:34 $ - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        test4c.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * main                 
 *	control_c				CTLC handler
 *
 * Notes:			This sample program uses the device driver to access the
 *						Modbus Plus network to read 125 holding registers from a
 *						984-x85 controller at network address given as an input.
 *
 * Author:     
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.1 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 *
 *=fdoc=====================================================================*/

#include <stdio.h>
#include <dos.h>
#include <process.h>
#include <string.h>
#include <conio.h>
#include <signal.h>
#include <netbios.h>
#include <netlib.h>


/*
**	prototypes
*/

int main(int argc, char *argv[]);
void control_c(void);


/*
**	global variables
*/

char mbuffer[256];
int completed;


/*=doc========================================================================
 *
 * Function:    main
 *
 * Description: 	Parses user input and gets register input.
 *
 * Returns:     
 *
 * Parameters:  	None
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int main(int argc, char *argv[])
{
	NCB *nd;
	int i;
	char Path[30];
	int rem_addr;
	int ret_val;

    if( argc < 2 ) {
        printf( "Usage is: A>TEST4C <slave node>\n");
        exit( 1 );
    }
    ret_val = sscanf( argv[ 1 ], "%d", &rem_addr );
    if( ret_val != 1 || i < 1 || rem_addr > 64 ) {
        printf( "Node number must be in the range 1 to 64. " );
        exit( 1 );
    }

    if( signal( SIGINT, control_c ) == SIG_ERR ) {
        printf( "Unable to redirect Control-C." );
        exit( 1 );
    }
   sprintf (Path, "DM.%d.1.0.0.0",rem_addr);

	if ((nd = ncb_open(Path, 0)) == NULL) {
		printf("Unable to open DATA MASTER path.\n");
		exit(1);
		}
	printf("Path %02X opened\n", nd->NCB_NUM);
	printf("Routing info: %c%c.%d.%d.%d.%d.%d\n",
		nd->NCB_CALLNAME[0],
		nd->NCB_CALLNAME[1],
		nd->NCB_CALLNAME[2],
		nd->NCB_CALLNAME[3],
		nd->NCB_CALLNAME[4],
		nd->NCB_CALLNAME[5],
		nd->NCB_CALLNAME[6]);

	while( !kbhit() ) {
		mbuffer[0] = 0x5;	/*slave address*/
		mbuffer[1] = 0x3;	/*command*/
		mbuffer[2] = 0x0;	/*offset high*/
		mbuffer[3] = 0x0;	/*offset low*/
		mbuffer[4] = 0x0;	/*reg count high*/
		mbuffer[5] = 125;	/* reg count low*/
		printf("+");
		if (ncb_send(nd, 6, mbuffer, 10) != 0) {
						/*send the command*/
			printf("Send error: %d.\n", nd->NCB_RETCODE);
			ncb_close(nd);	/*close the path*/
			exit(1);
			}
		if (ncb_receive_wait(nd, mbuffer, 10) != 0) {
						/*try to receive*/
			printf("\nReceive error: %d.\n", nd->NCB_RETCODE);
			}
		printf("-");
		}
	ncb_close(nd);		/*close the path*/

	exit(0);

	return 0;
	}


/*=doc========================================================================
 *
 * Function:    control_c
 *
 * Description: 	This routine replaces the control-C handler supplied by DOS.
 *						When the user enters a control_C, the program will vector to
 *						this routine, which will set the completed flag to non-zero.
 *						In this test program, this will cause the infinite loop
 *						within main to terminate.
 *
 * Returns:     None
 *
 * Parameters:  Signal which caused the call. (SIGINT)
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

void control_c()
{
    signal( SIGINT, SIG_IGN );          /* disable control-c */
    completed = 1;                      /* will cause main loop to complete */
    signal( SIGINT, control_c );        /* reset control-c handler */
}

