/*============================================================================
 *      MODICON, Inc., 1 High Street, North Andover, MA 01845, USA
 *      Copyright (c) $Date: 94/05/02 15:14:32 $ - All rights reserved.
 *
 *      No part of this document may be reproduced in any form
 *      without the express written consent of MODICON, Inc.
 *
 *=fdoc=======================================================================
 *
 * File:        testslav.c
 *
 * Functions:
 * Name                 Description
 * -------------------- -----------
 * main                   
 *	control_c				CTLC handler
 *	allocate_paths
 *	fslave6
 *	hi
 *	lo
 *	send_exc_response
 *	decode_modbus_msg
 *	slave_read_0x
 *	slave_read_1x
 *	slave_read_3x
 *	slave_read_4x
 *	slave_force_0x
 *	slave_force_0x_blk
 *	slave_preset_4x
 *	slave_preset_4x_blk
 *	slave_read_exc_status
 *	slave_loopback
 *	slave_slave_id
 *	slave_wcmd
 *	slave_rrep
 *	back_dbread
 *	back_dbwrite
 *	
 * Notes:		This program is an example of how to write a modbus slave
 *					application. The program initializes and opens one of the data
 *					slave input paths on the first MB+ adapter installed in the
 *					system.  When a modbus message is received from this path,
 *					this program will interpret the command, and generate the
 *					appropriate response.
 *
 *					Run this against test4c.
 *
 * Author:     
 *
 * RCSID:
 *
 * RCS:
 * $Revision: 1.1 $
 * $State: R3_41 $
 * $Locker: $
 *
 * Revision History Log:
 *
 *
 *=fdoc=====================================================================*/

#include <stdio.h>
#include <conio.h>
#include <dos.h>
#include <time.h>
#include <process.h>
#include <string.h>
#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <signal.h>
#include <netbios.h>
#include <netlib.h>
#include "testslav.h"

#define RETURN_GOOD_STATUS  0           /*library call success*/
#define RETURN_BAD_STATUS   -1          /*library call error*/


/*
**	prototypes
*/

int main(void);
void control_c(void);
int allocate_paths(unsigned char address);

int fslave6(unsigned int tblbuff[], unsigned char cmdbuff[], 
            unsigned char repbuff[], unsigned int dbuff[]);

extern int hi(unsigned int arg);
extern int lo(unsigned int arg);
extern int send_exc_response(int code, unsigned char *cmdbuff, unsigned char *repbuff);
extern int decode_modbus_msg(unsigned char *cmdbuff, unsigned char *repbuff, unsigned int *dbuff);
extern int slave_read_0x(void);
extern int slave_read_1x(void);
extern int slave_read_4x(unsigned char *cmdbuff, unsigned char *repbuff, unsigned int *dbuff);
extern int slave_read_3x(void);
extern int slave_force_0x(void);
extern int slave_preset_4x(unsigned char *cmdbuff, unsigned char *repbuff);
extern int slave_read_exc_status(void);
extern int slave_loopback(void);
extern int slave_force_0x_blk(void);
extern int slave_preset_4x_blk(unsigned char *cmdbuff, unsigned char *repbuff);
extern int slave_slave_id(void);
extern int slave_wcmd(int nbytes, unsigned char *cmdbuff);
extern int slave_rrep(int *n_bytes, unsigned char *repbuff);
extern int back_dbread(int offset, unsigned far *buffer, int quantity);
extern int back_dbwrite(int offset, unsigned far *buffer, int quantity);

unsigned int this_database[10000];
NCB data_ncb;
NCB *data_ncbp;

struct dxparamst DMA_area;
unsigned int *T = (unsigned int *) &DMA_area.table[0];
unsigned char cmdbuff[524], repbuff[524];
unsigned int dbuff[256];
int completed;
		 
static int swap(unsigned int *, int);
static int clength;		

/*=doc========================================================================
 *
 * Function:   main
 *
 * Description: Main processing loop
 *
 * Returns:     
 *
 * Parameters:  None
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int main()
	{
    int peer_number;                    /* this node address            */

	printf("\nAEG Modicon slave module for Modbus Plus 4/18/89");
	printf("\nhard-coded for Slaveid = 12\n");


	ncb_reset(0);		/* start by resetting the board */

    if( signal( SIGINT, control_c ) == SIG_ERR ) {
        printf( "Unable to redirect Control-C." );
        exit( 1 );
    }
    data_ncbp = &data_ncb;    
    data_ncbp->NCB_BUFFER = &repbuff[ 0 ];
                                       
    if( ncb_status( data_ncbp, 0 ) != 0 ) {   /* check to see if board present */
        printf( "Modbus Plus board not found.\n" );
        exit( 1 );
    }
    else {
        peer_number = ( int ) repbuff[ 6 ];
        printf( "Modbus Plus board identified as node address %d.\n",
            peer_number );
    }

	if ((data_ncbp = ncb_open("DS.1.0.0.0.0", 0)) == NULL) {
		printf("\nUnable to open DATA SLAVE path #1 (0x41 internal).\n");
		exit(1);
	}

    completed = 0;
	while( !completed ) {
        if( kbhit() )
            completed = 1;
        /* loop repeatedly */

		T[0] = 6;		    /* port "6" is Modbus Plus */
		T[1] = peer_number;		    /* Our slave address */
		T[2] = T[3] = 0;	/* don't use these, set them to zero */
		T[4] = 0;		    /* doesn't matter */
		fslave6(T, cmdbuff, repbuff, dbuff);

		if (T[5] != 0)
			printf("\nSlave Status = 0x%X", T[5]);
		else 
			printf("+");
    }
     /*
      * Completion Status is returned in T[DXSTATUS] (also returned by
      * the function), and also the detailed status is contained in
      * T[5].
      */

          exit( 0 );
          return 0;
	}

/*=doc========================================================================
 *
 * Function:    fslave6
 *
 * Description:	Simulate Modbus Slave
 *  					This routine will look for a Modbus message on the 
 *  					communications port. If a good (supported) Modbus message is
 *						received, and we're the correct slave, the approprate
 *						Modbus slave function will be called.
 *  					Current status will be returned in tblbuff[5].
 *  					Received Modbus function number will be returned in
 *						tblbuff[6].
 *
 * Returns:     	DXDONE if good MODBUS message, DXERROR otherwise.
 *
 * Parameters:  	Call block table of registers.
 *						command buffer
 *						reply buffer.
 *						data buffer.
 *
 * Notes:      
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/



int fslave6(tblbuff, cmdbuff, repbuff, dbuff)
unsigned int tblbuff[];
unsigned char repbuff[], cmdbuff[];
unsigned int dbuff[];
{
/*    char error_text[80];   goes with decode_status - not needed here */

/* CALL block table of registers: register offsets                      */
/* tblbuff[0] PORT          0     Modbus slave port to use (1-6) on Co-Pro  */
/* tblbuff[1] SLAVE         1     USER: Slave # we'll pretend to be         */
/* tblbuff[2] ADDRESS       2     not used  (in master, USER: Slave data address) */
/* tblbuff[3] QUANTITY      3     not used  (in master, USER: quantity of regs in/out */
/* tblbuff[4] MODE          4     not used  (in master, USER: 0 = ASCII  1 = RTU mode(relaxed timing only)) */
/* tblbuff[5] M_STATUS      5     Modbus slave status word - statuses appear here */
/* tblbuff[6] DATA1         6     function number for Modbus message received */
/*            (all other data regs follow)                              */

	static int nbytes;	/* # bytes read from slave      */

	tblbuff[M_STATUS] = slave_rrep(&nbytes, repbuff);

	if (tblbuff[M_STATUS] == 0) {
        /* now decode the good Modbus message */
		clength = nbytes;
		decode_modbus_msg(cmdbuff, repbuff, dbuff);
                          /* pass address, Co-Pro Port to use */
		tblbuff[DATA1] = repbuff[DATA1];	/* received Modbus function_no number is returned in tblbuff[1] */
		return DXDONE;
		}
	else 
		return DXERROR;	/* Bad Modbus message received */
}

/*=doc========================================================================
 *
 * Function:    decode_modbus_msg
 *
 * Description: 	Decode the MOBBUS message and send an appropriate response.
 *						Only commands dealing with the 4X registers are implemented.
 *
 * Returns:     	Always returns SUCCESS
 *
 * Parameters:  	command buffer
 *						reply buffer.
 *						data buffer.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/



int decode_modbus_msg(cmdbuff, repbuff, dbuff)
unsigned char *repbuff, *cmdbuff;
unsigned int *dbuff;
	{					/* decode_modbus_msg */

	switch (repbuff[1]) {

	case 1:				/* Modbus code 01 - read output status */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_read_0x(); */
		break;

	case 2:				/* Modbus code 02 - read input status */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_read_1x(); */
		break;

	case 3:				/* Modbus code 03 - read output registers */
		return (slave_read_4x(cmdbuff, repbuff, dbuff));

	case 4:				/* Modbus code 04 - read input registers */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_read_3x(); */
		break;

	case 5:				/* Modbus code 05 - force a single coil */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
		break;

	case 6:				/* Modbus code 06 - preset single register */
		slave_preset_4x(cmdbuff, repbuff);
		break;

	case 7:				/* Modbus code 07 - read slave exception coils */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_read_exc_status(); */
		break;

	case 8:				/* Modbus code 08 - loop back test */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_loopback(); */
		break;

	case 15:			/* Modbus code 15 - force multiple coils */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_force_0x_blk(); */
		break;

	case 16:			/* Modbus code 16 - preset multiple registers */
		slave_preset_4x_blk(cmdbuff, repbuff);
		break;

	case 17:			/* Modbus code 17 - report slave id */
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
/*      slave_slave_id(); */
		break;

	default:
		send_exc_response(1, cmdbuff, repbuff);
                        /* bad function, until implemented */
		break;
		}
	return SUCCESS;
	}					/* decode_modbus_msg */


/*=doc========================================================================
 *
 * Function:    slave_read_4x
 *
 * Description: Function to handle request for holding registers 
 *
 * Returns:     SUCCESS or FAILURE
 *
 * Parameters:  	command buffer
 *						reply buffer.
 *						data buffer.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int slave_read_4x(cmdbuff, repbuff, dbuff)
unsigned char repbuff[], cmdbuff[];
unsigned int dbuff[];
	{					/* slave_read_4x */
	static int i, j, offset, quantity;


/*
 * We Received:        |    We will send:
 * ------------------------------------------------------
 * address             |    address
 * function            |    function
 * data start reg high |    byte count
 * data start reg low  |    data out reg high  40108
 * data # regs high    |    data out reg low   40108
 * data # regs low     |    data out reg high  40109
 *                     |    data out reg low   40109
 *                     |    etc
 */
	swap((unsigned int *) &repbuff[2], 2);	/* fixup address and quantity */
	offset = *(int *) &repbuff[2];
	quantity = *(int *) &repbuff[4];	/* quantity in 16-bit WORDS, not bytes */

/* validate quantity here. If greater than 125 registers, send back exception response */
	if (quantity > 125) {
		send_exc_response(3, cmdbuff, repbuff);
                        /* bad data */
		return FAILURE;
		}
	else {
		if (back_dbread(offset, dbuff, quantity) == 0) {
                        /* int offset, unsigned int *buffer, int qty */

			cmdbuff[0] = repbuff[0];
                        /* byte 1 is our slave # */
			cmdbuff[1] = repbuff[1];
                        /* byte 2 is function code */
			cmdbuff[2] = (unsigned char) quantity * 2;
                        /* byte 3 is byte count */

        /* Note: the qty field in the next line was for REGISTERS, so we have to multiply by 2 */
			for (i = 3,  j = 0;  i < (3 + (2 * quantity));  i += 2,  ++j) {
				cmdbuff[i] = (unsigned char) hi(dbuff[j]);
				cmdbuff[i + 1] = (unsigned char) lo(dbuff[j]);
				}
			slave_wcmd(i, cmdbuff);
                        /* transmit response */
			return SUCCESS;
			}
		else {
/* Also, transmit back exception response here */
			send_exc_response(2, cmdbuff, repbuff);
                        /* bad address */
			return FAILURE;
			}
		}
	}					/* slave_read_4x */

/*=doc========================================================================
 *
 * Function:    slave_preset_4x
 *
 * Description: 	Function to handle request for setting a single
 *						holding register.
 *
 * Returns:     SUCCESS or FAILURE
 *
 * Parameters:  	command buffer
 *						reply buffer.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int slave_preset_4x(cmdbuff, repbuff)
unsigned char repbuff[], cmdbuff[];
	{					/* slave_preset_4x */
	static int offset;
	static unsigned int data;
	int broadcast;		/* if 1, broadcast mode, send no response */


/*
 * We Received:        |    We will send:
 * ------------------------------------------------------
 * address             |    address
 * function            |    function
 * data reg high       |    data reg high
 * data reg low        |    data reg low 
 * data value high     |    data value hight
 * data value low      |    data value low
 * (In this case response identical to what we received)
 */
	swap((unsigned int *) &repbuff[2], 2);	/* fixup address and data */
	broadcast = !repbuff[0];	/* if broadcast mode, address is 0 */
	offset = *(int *) &repbuff[2];
	data = *(int *) &repbuff[4];

	if (back_dbwrite(offset, &data, 1) == 0) {
                        /* int offset, unsigned int *buffer, int qty */
		if (!broadcast) {
                        /* If not Broadcast mode, issue response */
			cmdbuff[0] = repbuff[0];
                        /* byte 1 is our slave # */
			cmdbuff[1] = repbuff[1];
                        /* byte 2 is function code */
			cmdbuff[2] = (unsigned char) hi(offset);
                        /* byte 3 is offset (ho) */
			cmdbuff[3] = (unsigned char) lo(offset);
                        /* byte 4 is offset (lo) */
			cmdbuff[4] = (unsigned char) hi(data);
                        /* byte 5 is data (ho) */
			cmdbuff[5] = (unsigned char) lo(data);
                        /* byte 5 is data (lo) */

			slave_wcmd(6, cmdbuff);
                        /* transmit response if not Broadcast mode */
			}
		return SUCCESS;
		}
	else {
/* Also, transmit back exception response here if not Broadcast mode */
		if (!broadcast)
			send_exc_response(2, cmdbuff, repbuff);
                        /* bad address */
		return FAILURE;
		}
	}					/* slave_preset_4x */


/*=doc========================================================================
 *
 * Function:    slave_preset_4x_blk
 *
 * Description: 	Function to handle request for setting multiple
 *						holding registers.
 *
 * Returns:     SUCCESS or FAILURE
 *
 * Parameters:  	command buffer
 *						reply buffer.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int slave_preset_4x_blk(cmdbuff, repbuff)
unsigned char repbuff[], cmdbuff[];
	{					
	static int offset, quantity;
	static unsigned char *workptr;
	int broadcast;		/* if 1, broadcast mode, send no response */


/*
 * We Received:        |    We will send:
 * ------------------------------------------------------
 * address             |    address
 * function            |    function
 * reg addr high       |    reg addr high
 * reg addr low        |    reg addr low 
 * data quantity high  |    data quantity high
 * data quantity low   |    data quantity low
 * byte count          |
 * data high    reg 1  |
 * data low            |
 * data high    reg 2  |
 * data low            |
 */
	swap((unsigned int *) &repbuff[2], 2);	/* fixup address and quantity */
	offset = *(int *) &repbuff[2];
	broadcast = !repbuff[0];	/* If broadcast mode, address is 0 */
	quantity = *(int *) &repbuff[4];	/* quantity in 16-bit WORDS, not bytes */

/*
 * validate quantity here.
 * If greater than 125 registers,
 * and not broadcast mode,
 * send back exception response
 */
	if ((quantity > 125) && (!broadcast)) {
		send_exc_response(3, cmdbuff, repbuff);
                        /* bad data */
		return FAILURE;
		}
	else {
		workptr = (unsigned char *) &repbuff[0];
		workptr += 7;	/* skip addr/func/haddr/laddr/hqty/lqty/bytecount */

		swap((unsigned int *) workptr, quantity);
            /* swap database databytes */

		if (back_dbwrite(offset, (unsigned int *) workptr, quantity) == 0) {
                        /* int offset, unsigned int *buffer, int quantity */

			if (!broadcast) {
                        /* If not Broadcast mode, issue response */
				cmdbuff[0] = repbuff[0];
                        /* byte 1 is our slave # */
				cmdbuff[1] = repbuff[1];
                        /* byte 2 is function code */
				cmdbuff[2] = (unsigned char) hi(offset);
                        /* byte 3 is offset (ho)*/
				cmdbuff[3] = (unsigned char) lo(offset);
                        /* byte 4 is offset (lo) */
				cmdbuff[4] = (unsigned char) hi(quantity);
                        /* byte 5 is quantity (ho) */
				cmdbuff[5] = (unsigned char) lo(quantity);
                        /* byte 6 is quantity (lo) */

				slave_wcmd(6, cmdbuff);
                        /* transmit response */
				}
			return SUCCESS;
			}
		else {
                            /* Also, transmit back exception response here */
			if (!broadcast)	/* If not Broadcast mode, issue response */
				send_exc_response(2, cmdbuff, repbuff);
                        /* bad address */
			return FAILURE;
			}
		}
	}					/* slave_preset_4x_blk */


/*=doc========================================================================
 *
 * Function:    swap
 *
 * Description:  Swap hi and low bytes of an array of integers.
 *
 * Returns:     Always return SUCCESS
 *
 * Parameters:  	Pointer to the array and the number of ints to swap
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/



static int swap(T, qty)
unsigned int T[];
int qty;
	{				 
	int i;
	unsigned int cswap;

	for (i = 0;  i < qty;  ) {
		cswap = T[i];
		cswap >>= 8;
		T[i] <<= 8;
		T[i++] |= cswap;
		}
	return SUCCESS;
	}					/* swap */

/*=doc========================================================================
 *
 * Function:    send_exc_response
 *
 * Description: Send exception response 
 *
 * Returns:     	Always returns SUCCESS
 *
 * Parameters:		error code to send to the master.
 *						command buffer.
 *						reply buffer.
 *						
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/



int send_exc_response(code, cmdbuff, repbuff)
int code;				/* Error  code */
/*
 * Error Codes:
 * 1     Illegal Function
 * 2     Illegal Data Address
 * 3     Illegal Data Value
 * 4     Failure in associated device
 * 5     Acknowledge
 * 6     Busy, rejected message
 * 7     Nak-Negative acknowledgement
 */
unsigned char repbuff[], cmdbuff[];
	{					/* send_exc_response */
	cmdbuff[0] = (unsigned char) repbuff[0];
                        /* byte 1 is our slave # */
	cmdbuff[1] = (unsigned char) repbuff[1] | 0x80;
                        /* byte 2 is function code plus exception bit */
	cmdbuff[2] = (unsigned char) code;	/* Error code */
	slave_wcmd(3, cmdbuff);
                        /* transmit response */
	return SUCCESS;
	}					/* send_exc_response */


/*=doc========================================================================
 *
 * Function:    back_dbwrite
 *
 * Description: 	This procedure writes to the database structure which is
 *						contained in memory.
 *
 * Returns:     None
 *
 * Parameters:  	offset into the database.
 *						buffer to write.
 *						number of words to write.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int back_dbwrite(offset, buffer, quantity)
int offset;
unsigned far *buffer;
int quantity;
	{
	long db_size;		/*size of database in words*/

	if (quantity > 255)
		return (RETURN_BAD_STATUS);	/*return error*/

	db_size = (sizeof (this_database) * 2L);
                        /*get size in words (8 words/paragraph)*/

	if (((long) offset + (long) quantity) > db_size)
		return (RETURN_BAD_STATUS);	/*not entirely within boundaries of database*/

	memmove((unsigned char *) this_database + (offset * 2), buffer, quantity * 2);

	return (RETURN_GOOD_STATUS);	/*return good status*/
	}


/*=doc========================================================================
 *
 * Function:    back_dbread
 *
 * Description: 	This procedure reads from the database structure which is
 *						contained in memory.
 *
 * Returns:     None
 *
 * Parameters:  	None
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int back_dbread(offset, buffer, quantity)
int offset;
unsigned far *buffer;
int quantity;
	{
	long db_size;		/*size of database in words*/

	if (quantity > 255)
		return (RETURN_BAD_STATUS);	/*return error*/

	db_size = (sizeof (this_database) * 2L);
                        /*get size in words (8 words/paragraph)*/

	if (((long) offset + (long) quantity) > db_size)
		return (RETURN_BAD_STATUS);	/*not entirely within boundaries of database*/

	memmove(buffer, (unsigned char *) this_database + (offset * 2), quantity * 2);

	return (RETURN_GOOD_STATUS);	/*return good status*/
	}

/*=doc========================================================================
 *
 * Function:    slave_wcnd
 *
 * Description: 	Write given sequence of command bytes to slave network,
 *						and append appropriate type of checksum (LRC or CRC) to
 *						the message end.
 *
 * Returns:     SUCCESS or failure (0x204)
 *
 * Parameters:		number of bytes to write.
 *						command buffer
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int slave_wcmd(nbytes, cmdbuff)
int nbytes;				/* number of bytes to write     */
unsigned char cmdbuff[];	/* buffer of bytes to send      */
	{


	if (ncb_send(data_ncbp, nbytes, cmdbuff, 30) != 0) {
                        /*send the command*/
		printf("Send error: %d.\n", data_ncbp->NCB_RETCODE);
		return 0x204;	/* "failure in associated device" */
		}
	else 
		return 0;

	}

/*=doc========================================================================
 *
 * Function:    slave_rrep
 *
 * Description: read slave reply 
 *
 * Returns:     SUCCESS or failure (0x204)
 *
 * Parameters:  	Pointer a place to store the number of bytes read.
 *						Pointer to a buffer for the data which is read.
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int slave_rrep(n_bytes, repbuff)
int *n_bytes;			/* # of data bytes (ex chksum) read by rrep */
unsigned char repbuff[];	/* buffer to store the reply    */
	{

	*n_bytes = 0;		/* initially set n_bytes to 0 */

	if (ncb_receive_wait(data_ncbp, repbuff, 30) != 0) {
                        /*try to receive*/
		*n_bytes = 0;
		return 0x204;	/* "failure in associated device" */
		}
	else {
		*n_bytes = data_ncbp->NCB_LENGTH;
		return 0;
		}
	}

/*=doc========================================================================
 *
 * Function:    hi
 *
 * Description: returns high order byte (in low order bits) of argument
 *
 * Returns:   
 *
 * Parameters:  
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

int hi(arg)
unsigned int arg;
	{
	return ((arg & 0xFF00) >> 8);
	}

/*=doc========================================================================
 *
 * Function:    lo
 *
 * Description: returns low order byte (in low order bits) of argument
 *
 * Returns:   
 *
 * Parameters:  
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/


int lo(arg)
unsigned int arg;
	{
	return (arg & 0xFF);
	}



/*=doc========================================================================
 *
 * Function:    control_c
 *
 * Description: 	This routine replaces the control-C handler supplied by DOS.
 *						When the user enters a control_C, the program will vector to
 *						this routine, which will set the completed flag to non-zero.
 *						In this test program, this will cause the infinite loop
 *						within main to terminate.
 *
 * Returns:     None
 *
 * Parameters:  Signal which caused the call. (SIGINT)
 *
 * Notes:       
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/

void control_c()
{
    signal( SIGINT, SIG_IGN );          /* disable control-c */
    completed = 1;                      /* will cause main loop to complete */
    signal( SIGINT, control_c );        /* reset control-c handler */
}


/*=doc========================================================================
 *
 * Function:    Currently unimplemented command service routines
 *
 * Description: x
 *
 * Returns:     x
 *
 * Parameters:  Pointer to NCB structure.
 *
 * Notes:       x
 *
 * Revision History Log:
 *
 *	NAME					DATE			COMMENT
 *	-----					-----			--------
 *=doc======================================================================*/



int slave_slave_id()
	{					
	return SUCCESS;
	}

int slave_read_0x()
	{					
	return SUCCESS;
	}					

int slave_read_1x()
	{					
	return SUCCESS;
	}					

int slave_read_3x()
	{					
	return SUCCESS;
	}					

int slave_force_0x()
	{					
	return SUCCESS;
	}					

int slave_read_exc_status()
	{					
	return SUCCESS;
	}					

int slave_loopback()
	{					
	return SUCCESS;
	}					

int slave_force_0x_blk()
	{					
	return SUCCESS;
	}					

/*********************** end of testslav.c ************************/
